/*-
 * Copyright (c) 2011-2012 Tuxera Inc.
 *
 * See COPYING.txt for license details.
 */

/*
 * Copyright (C) 2006-2008 Google. All Rights Reserved.
 * Amit Singh <singh@>
 */

#include "fuse_kludges.h"
#include <libkern/version.h>

#if M_MACFUSE_ENABLE_EXCHANGE

extern void
fuse_kludge_exchange(vnode_t v1, vnode_t v2)
{
    if (version_major > 10) {
        char *tmp_v_name = ((struct fuse_kludge_vnode_11 *)v1)->v_name;
        ((struct fuse_kludge_vnode_11 *)v1)->v_name =
            ((struct fuse_kludge_vnode_11 *)v2)->v_name;
        ((struct fuse_kludge_vnode_11 *)v2)->v_name = tmp_v_name;

        vnode_t tmp_v_parent = ((struct fuse_kludge_vnode_11 *)v1)->v_parent;
        ((struct fuse_kludge_vnode_11 *)v1)->v_parent =
            ((struct fuse_kludge_vnode_11 *)v2)->v_parent;
        ((struct fuse_kludge_vnode_11 *)v2)->v_parent = tmp_v_parent;
    }
    else if (version_major > 9) {
        char *tmp_v_name = ((struct fuse_kludge_vnode_10 *)v1)->v_name;
        ((struct fuse_kludge_vnode_10 *)v1)->v_name =
            ((struct fuse_kludge_vnode_10 *)v2)->v_name;
        ((struct fuse_kludge_vnode_10 *)v2)->v_name = tmp_v_name;

        vnode_t tmp_v_parent = ((struct fuse_kludge_vnode_10 *)v1)->v_parent;
        ((struct fuse_kludge_vnode_10 *)v1)->v_parent =
            ((struct fuse_kludge_vnode_10 *)v2)->v_parent;
        ((struct fuse_kludge_vnode_10 *)v2)->v_parent = tmp_v_parent;
    } else if (version_major > 8) {
        char *tmp_v_name = ((struct fuse_kludge_vnode_9 *)v1)->v_name;
        ((struct fuse_kludge_vnode_9 *)v1)->v_name =
            ((struct fuse_kludge_vnode_9 *)v2)->v_name;
        ((struct fuse_kludge_vnode_9 *)v2)->v_name = tmp_v_name;

        vnode_t tmp_v_parent = ((struct fuse_kludge_vnode_9 *)v1)->v_parent;
        ((struct fuse_kludge_vnode_9 *)v1)->v_parent =
            ((struct fuse_kludge_vnode_9 *)v2)->v_parent;
        ((struct fuse_kludge_vnode_9 *)v2)->v_parent = tmp_v_parent;
    } else {
        char *tmp_v_name = ((struct fuse_kludge_vnode_8 *)v1)->v_name;
        ((struct fuse_kludge_vnode_8 *)v1)->v_name =
            ((struct fuse_kludge_vnode_8 *)v2)->v_name;
        ((struct fuse_kludge_vnode_8 *)v2)->v_name = tmp_v_name;

        vnode_t tmp_v_parent = ((struct fuse_kludge_vnode_8 *)v1)->v_parent;
        ((struct fuse_kludge_vnode_8 *)v1)->v_parent =
            ((struct fuse_kludge_vnode_8 *)v2)->v_parent;
        ((struct fuse_kludge_vnode_8 *)v2)->v_parent = tmp_v_parent;
    }
}

#endif /* M_MACFUSE_ENABLE_EXCHANGE */

#if VERSION_MAJOR <= 8
#define MNT_KERN_FLAG_OFFSET 64

#define MNTK_LOCK_LOCAL 0x00100000
#define MNTK_UNMOUNT    0x01000000

void
vfs_setlocklocal(mount_t mp)
{
    /*
     * Horrible, horrible kludge. Dangerous to boot. "Boot", heh.
     * The issue is that we really need to do vfs_setlocklocal(mp),
     * otherwise we won't have VFS-provided advisory file locking.
     * Since the kernel's extended attributes needs to create files
     * with O_EXLOCK set, we need advisory locking for extended
     * attributes to work properly. Since ACLs depend on extended
     * attributes, vfs_setlocklocal(mp) keeps becoming critical.
     *
     * The kludge is, well, just setting the flag "by hand". This
     * means I'm hardcoding the offset of the flag word field in the
     * mount structure, which is internal to the kernel (not exposed
     * through the KPIs).
     *
     * If the field offset changes in future (not likely, but not
     * guaranteed to remain the same either), the following operation
     * could result in weird behavior, including a kernel panic.
     *
     */
    *(int *)((char *)mp + MNT_KERN_FLAG_OFFSET) |= MNTK_LOCK_LOCAL;
}
#endif /* VERSION_MAJOR > 8 */

static inline lck_mtx_t* fuse_kludge_vnode_get_v_lock(vnode_t vp)
{
    if(version_major >= 11)
        return (lck_mtx_t*) (((struct fuse_kludge_vnode_11*) vp)->v_lock);
    else if(version_major >= 10)
        return (lck_mtx_t*) (((struct fuse_kludge_vnode_10*) vp)->v_lock);
    else if(version_major >= 9)
        return (lck_mtx_t*) (((struct fuse_kludge_vnode_9*) vp)->v_lock);
    else
        return (lck_mtx_t*) (((struct fuse_kludge_vnode_8*) vp)->v_lock);
}

static inline uint16_t fuse_kludge_vnode_get_v_lflag(vnode_t vp)
{
    if(version_major >= 11)
        return ((struct fuse_kludge_vnode_11*) vp)->v_lflag;
    else if(version_major >= 10)
        return ((struct fuse_kludge_vnode_10*) vp)->v_lflag;
    else if(version_major >= 9)
        return ((struct fuse_kludge_vnode_9*) vp)->v_lflag;
    else
        return ((struct fuse_kludge_vnode_8*) vp)->v_lflag;
}

/* Constants from vnode_internal.h. */
#define FUSE_KLUDGE_VL_TERMINATE    0x0004
#define FUSE_KLUDGE_VL_DEAD         0x0010
__private_extern__ int fuse_kludge_vnode_isrecycled(vnode_t vp)
{
    lck_mtx_t *v_lock;
    uint16_t v_lflag;

    v_lock = fuse_kludge_vnode_get_v_lock(vp);

    /* Note: In 10.5 and later vnode_isrecycled uses lck_mtx_lock_spin for
     * i386/x86_64, but this is kernel private so we can only use 'regular'
     * lck_mtx_lock. */
    lck_mtx_lock(v_lock);
    v_lflag = fuse_kludge_vnode_get_v_lflag(vp);
    lck_mtx_unlock(v_lock);

    return (v_lflag & (FUSE_KLUDGE_VL_TERMINATE | FUSE_KLUDGE_VL_DEAD)) ? 1 : 0;
}

#if VERSION_MAJOR == 8
struct fuse_kludge_mount_8 {
    void *dummy[2];
    int dummy2[1 + 3];
    void *dummy3;
    struct fuse_kludge_vfstable_8 *mnt_vtable;
    char dummy4[0];
};

struct fuse_kludge_vfstable_8 {
    void *dummy1;
    char dummy2[15];
    int dummy3[3];
    void *dummy4[2];
    int dummy5[1 + 3];
    int vfc_vfsflags;
    char dummy6[0];
};

#define FUSE_KLUDGE_VFC_VFSLOCALARGS_8     0x02
#define FUSE_KLUDGE_VFC_VFSGENERICARGS_8   0x04
#define FUSE_KLUDGE_VFC_VFSNATIVEXATTR_8   0x10

/* Very dangerous 'kludge' relying on two struct member offsets being accurate.
 * It allows enabling native extended attributes for non-kernel filesystems in
 * Mac OS X 10.4.
 * The xnu kernel definitions for these structs are the same throughout the Mac
 * OS X 10.4 kernel series, but it has only been really verified on PowerPC
 * (because I don't have an Intel Mac that can run Mac OS X 10.4 anymore).
 * Hopefully it works on Intel too, eh?
 *
 *   Erik Larsson, 2012-01-31
 */
void fuse_kludge_darwin8_set_nativexattr(mount_t mp)
{
    struct fuse_kludge_mount_8 *const mp_v8 = (struct fuse_kludge_mount_8*) mp;

    if(version_major != 8) {
        IOLog("%s: Invalid kernel version %d!\n",
              __FUNCTION__, version_major);
        return;
    }

    if(mp_v8->mnt_vtable->vfc_vfsflags ==
       (FUSE_KLUDGE_VFC_VFSGENERICARGS_8 | FUSE_KLUDGE_VFC_VFSNATIVEXATTR_8))
    {
        /* The VFC_VFSNATIVEXATTR flag is already set for mnt_vtable.
         * Since there is one mnt_vtable per registered filesystem rather than
         * per mount, this condition will happen for all mounts except the first
         * mount after the kext is loaded into the kernel. */
    }
    else if(mp_v8->mnt_vtable->vfc_vfsflags == FUSE_KLUDGE_VFC_VFSGENERICARGS_8)
    {
        mp_v8->mnt_vtable->vfc_vfsflags |= FUSE_KLUDGE_VFC_VFSNATIVEXATTR_8;
    }
    else {
        IOLog("%s: Unexpected current value of vfc_vfsflags 0x%X!\n",
              __FUNCTION__, mp_v8->mnt_vtable->vfc_vfsflags);
    }
}
#endif /* VERSION_MAJOR == 8 */
